@@ -2,6 +2,7 @@
 #define BRANCHGROUP_H 1
 
 #include "Common/Algorithms.h"
+#include "Common/Exception.h"
 #include <algorithm> // for swap
 #include <map>
 #include <utility>
@@ -209,9 +210,4 @@ isAmbiguous(const SequenceCollectionHash& g) const
 		BranchGroupStatus m_status;
 };
 
-namespace std {
-	template <>
-	inline void swap(BranchGroup&, BranchGroup&) { assert(false); }
-}
-
 #endif
