@@ -7,6 +7,7 @@
 #include "Common/Kmer.h"
 #include "Common/BitUtil.h"
 #include "Common/KmerIterator.h"
+#include "Common/UnorderedSet.h"
 #include "Graph/Path.h"
 #include "Graph/ExtendPath.h"
 #include "Konnector/DBGBloom.h"
@@ -18,8 +19,11 @@
 #include "Bloom/CascadingBloomFilter.h"
 #include "Bloom/BloomFilterWindow.h"
 #include "Bloom/CascadingBloomFilterWindow.h"
+#include "Bloom/RollingBloomDBGVisitor.h"
 #include "BloomDBG/BloomIO.h"
 #include "BloomDBG/HashAgnosticCascadingBloom.h"
+#include "BloomDBG/RollingBloomDBG.h"
+#include "BloomDBG/RollingHashIterator.h"
 #include "lib/bloomfilter/BloomFilter.hpp"
 
 #include <cstdlib>
@@ -51,9 +55,11 @@ static const char USAGE_MESSAGE[] =
 "Usage 3: " PROGRAM " intersect [GLOBAL_OPTS] [COMMAND_OPTS] <OUTPUT_BLOOM_FILE> <BLOOM_FILE_1> <BLOOM_FILE_2> [BLOOM_FILE_3]...\n"
 "Usage 4: " PROGRAM " info [GLOBAL_OPTS] [COMMAND_OPTS] <BLOOM_FILE>\n"
 "Usage 5: " PROGRAM " compare [GLOBAL_OPTS] [COMMAND_OPTS] <BLOOM_FILE_1> <BLOOM_FILE_2>\n"
-"Usage 6: " PROGRAM " kmers [GLOBAL_OPTS] [COMMAND_OPTS] <BLOOM_FILE> <READS_FILE>\n"
-"Usage 7: " PROGRAM " trim [GLOBAL_OPTS] [COMMAND_OPTS] <BLOOM_FILE> <READS_FILE> [READS_FILE_2]... > trimmed.fq\n"
-"Build and manipulate bloom filter files.\n"
+"Usage 6: " PROGRAM " graph [GLOBAL_OPTS] [COMMAND_OPTS] <BLOOM_FILE>\n"
+"Usage 7: " PROGRAM " kmers [GLOBAL_OPTS] [COMMAND_OPTS] <BLOOM_FILE> <READS_FILE>\n"
+"Usage 8: " PROGRAM " trim [GLOBAL_OPTS] [COMMAND_OPTS] <BLOOM_FILE> <READS_FILE> [READS_FILE_2]... > trimmed.fq\n"
+"\n"
+"Build and manipulate Bloom filter files.\n"
 "\n"
 " Global options:\n"
 "\n"
@@ -98,6 +104,13 @@ static const char USAGE_MESSAGE[] =
 "  -m, --method=`String'      choose distance calculation method \n"
 "                             [`jaccard'(default), `forbes', `czekanowski']\n"
 "\n"
+" Options for `" PROGRAM " graph':\n"
+"\n"
+"  -d, --depth=N              depth of neighbouring from --root [k]\n"
+"  -R, --root=KMER            root k-mer from graph traversal [required]\n"
+"  -a, --node-attr=STR:FILE   assign a node attribute (e.g. 'color=blue')\n"
+"                             k-mers in the given FASTA\n"
+"\n"
 " Options for `" PROGRAM " kmers':\n"
 "\n"
 "  -r, --inverse              get k-mers that are *NOT* in the bloom filter\n"
@@ -112,6 +125,13 @@ static const char USAGE_MESSAGE[] =
 enum BloomFilterType { BT_KONNECTOR, BT_ROLLING_HASH, BT_UNKNOWN };
 enum OutputFormat { BED, FASTA, RAW };
 
+/* types related to --node-attr option */
+
+typedef string KmerProperty;
+typedef string FastaPath;
+typedef vector<pair<KmerProperty, FastaPath> > KmerProperties;
+typedef KmerProperties::iterator KmerPropertiesIt;
+
 namespace opt {
 
 	/** The size of the bloom filter in bytes. */
@@ -120,6 +140,9 @@ namespace opt {
 	/** The size of the I/O buffer of each thread, in bytes  */
 	size_t bufferSize = 100000;
 
+	/** Depth of graph traversal */
+	size_t depth = 0;
+
 	/** The number of parallel threads. */
 	unsigned threads = 1;
 
@@ -150,6 +173,13 @@ namespace opt {
 	/** The type of Bloom filter to build */
 	BloomFilterType bloomType = BT_KONNECTOR;
 
+	/**
+	 * For the "graph" command: assign node attribute
+	 * (e.g. "color=blue") to k-mers contained in the
+	 * associated FASTA file
+	 */
+	KmerProperties kmerProperties;
+
 	/** Index of bloom filter window.
 	  ("M" for -w option) */
 	unsigned windowIndex = 0;
@@ -168,10 +198,13 @@ namespace opt {
 	 */
 	bool inverse = false;
 
+	/** Root node (k-mer) for `graph` subcommand */
+	string root;
+
 	OutputFormat format = FASTA;
 }
 
-static const char shortopts[] = "b:B:h:H:j:k:l:L:m:n:q:rvt:w:";
+static const char shortopts[] = "a:b:B:d:h:H:j:k:l:L:m:n:q:rR:vt:w:";
 
 enum { OPT_HELP = 1, OPT_VERSION, OPT_BED, OPT_FASTA, OPT_RAW };
 
@@ -179,6 +212,7 @@ static const struct option longopts[] = {
 	{ "bloom-size", required_argument, NULL, 'b' },
 	{ "bloom-type", required_argument, NULL, 't' },
 	{ "buffer-size", required_argument, NULL, 'B' },
+	{ "depth", required_argument, NULL, 'd' },
 	{ "hash-seed", required_argument, NULL, 'h' },
 	{ "num-hashes", required_argument, NULL, 'H' },
 	{ "threads", required_argument, NULL, 'j' },
@@ -193,19 +227,22 @@ static const struct option longopts[] = {
 	{ "trim-quality", required_argument, NULL, 'q' },
 	{ "standard-quality", no_argument, &opt::qualityOffset, 33 },
 	{ "illumina-quality", no_argument, &opt::qualityOffset, 64 },
+	{ "node-attr", required_argument, NULL, 'a' },
 	{ "verbose", no_argument, NULL, 'v' },
 	{ "help", no_argument, NULL, OPT_HELP },
 	{ "version", no_argument, NULL, OPT_VERSION },
 	{ "window", required_argument, NULL, 'w' },
 	{ "method", required_argument, NULL, 'm' },
 	{ "inverse", required_argument, NULL, 'r' },
+	{ "root", required_argument, NULL, 'R' },
 	{ "bed", no_argument, NULL, OPT_BED },
 	{ "fasta", no_argument, NULL, OPT_FASTA },
 	{ "raw", no_argument, NULL, OPT_RAW },
 	{ NULL, 0, NULL, 0 }
 };
 
-void dieWithUsageError()
+__attribute__((noreturn))
+static void dieWithUsageError()
 {
 	cerr << "Try `" << PROGRAM
 		<< " --help' for more information.\n";
@@ -418,9 +455,9 @@ static inline void buildKonnectorBloom(size_t bits, string outputPath,
 	if (opt::windows == 0) {
 
 		if (opt::levels == 1) {
-			BloomFilter bloom(bits, opt::hashSeed);
+			Konnector::BloomFilter bloom(bits, opt::hashSeed);
 #ifdef _OPENMP
-			ConcurrentBloomFilter<BloomFilter>
+			ConcurrentBloomFilter<Konnector::BloomFilter>
 				cbf(bloom, opt::numLocks, opt::hashSeed);
 			loadFilters(cbf, argc, argv);
 #else
@@ -640,7 +677,7 @@ int combine(int argc, char** argv, BitwiseOp readOp)
 	string outputPath(argv[optind]);
 	optind++;
 
-	BloomFilter bloom;
+	Konnector::BloomFilter bloom;
 
 	for (int i = optind; i < argc; i++) {
 		string path(argv[i]);
@@ -695,7 +732,7 @@ int info(int argc, char** argv)
 		dieWithUsageError();
 	}
 
-	BloomFilter bloom;
+	Konnector::BloomFilter bloom;
 	string path = argv[optind];
 
 	if (opt::verbose)
@@ -744,9 +781,9 @@ int compare(int argc, char ** argv){
 	std::cerr << "Computing distance for 2"
 			  << " samples...\n";
 	// Get both paths and open istreams
-	BloomFilter bloomA;
+	Konnector::BloomFilter bloomA;
 	string pathA(argv[optind]);
-	BloomFilter bloomB;
+	Konnector::BloomFilter bloomB;
 	string pathB(argv[optind+1]);
 	if (opt::verbose)
 	  std::cerr << "Loading bloom filters from "
@@ -843,9 +880,114 @@ int compare(int argc, char ** argv){
   return 1;
 }
 
+int graph(int argc, char** argv)
+{
+	parseGlobalOpts(argc, argv);
+
+	// default graph neighbourhood depth
+	opt::depth = opt::k;
+
+	for (int c; (c = getopt_long(argc, argv,
+					shortopts, longopts, NULL)) != -1;) {
+		istringstream arg(optarg != NULL ? optarg : "");
+		switch (c) {
+		  case '?':
+			dieWithUsageError();
+		  case 'a':
+			{
+				string s;
+				arg >> s;
+				size_t pos = s.find(":");
+				if (pos < s.length())
+					opt::kmerProperties.push_back(make_pair(
+						s.substr(0, pos), s.substr(pos + 1)));
+				else
+					arg.setstate(ios::failbit);
+			}
+			break;
+		  case 'd':
+			arg >> opt::depth; break;
+		  case 'R':
+			arg >> opt::root; break;
+		}
+		if (optarg != NULL && (!arg.eof() || arg.fail())) {
+			cerr << PROGRAM ": invalid option: `-"
+				<< (char)c << optarg << "'\n";
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	if (opt::root.empty()) {
+		cerr << PROGRAM ": missing required option --root <KMER>\n";
+		dieWithUsageError();
+	}
+
+	if (opt::root.size() != opt::k) {
+		cerr << PROGRAM ": --root arg must be a k-mer of length "
+			<< opt::k << "\n";
+		dieWithUsageError();
+	}
+
+	if (argc - optind != 1) {
+		cerr << PROGRAM ": missing arguments\n";
+		dieWithUsageError();
+	}
+
+	typedef RollingBloomDBG<HashAgnosticCascadingBloom> Graph;
+	typedef boost::graph_traits<Graph>::vertex_descriptor V;
+
+	string bloomPath(argv[optind]);
+	optind++;
+
+	vector<pair<string, unordered_set<V> > > kmerProperties;
+	for (KmerPropertiesIt it = opt::kmerProperties.begin();
+		it != opt::kmerProperties.end(); ++it) {
+
+		if (opt::verbose)
+			cerr << "Loading k-mers from `" << it->second << "', to be "
+				<< "annotated with '" << it->first << "'\n";
+
+		kmerProperties.push_back(make_pair(it->first, unordered_set<V>()));
+
+		size_t count = 0;
+		size_t checkpoint = 0;
+		const size_t step = 10000;
+
+		FastaReader in(it->second.c_str(), FastaReader::FOLD_CASE);
+		for (FastaRecord rec; in >> rec;) {
+			for (RollingHashIterator it(rec.seq, 1, opt::k);
+				 it != RollingHashIterator::end(); ++it, ++count) {
+				V v(it.kmer().c_str(), it.rollingHash());
+				kmerProperties.back().second.insert(v);
+			}
+			while (opt::verbose && count >= checkpoint) {
+				cerr << "Loaded " << checkpoint << " k-mers\n";
+				checkpoint += step;
+			}
+		}
+		if (opt::verbose)
+			cerr << "Loaded " << count << " k-mers in total\n";
+	}
+
+	if (opt::verbose)
+		cerr << "Loading Bloom filter from `" << bloomPath << "'..." << endl;
+
+	HashAgnosticCascadingBloom bloom(bloomPath);
+	assert(opt::k == bloom.getKmerSize());
+
+	Graph g(bloom);
+	V root(opt::root.c_str(), RollingHash(opt::root.c_str(),
+		bloom.getHashNum(), bloom.getKmerSize()));
+
+	RollingBloomDBGVisitor<Graph> visitor(root, opt::depth, kmerProperties, cout);
+	breadthFirstSearch(root, g, true, visitor);
+
+	return 0;
+}
+
 int memberOf(int argc, char ** argv){
 	// Initalise bloom and get globals
-	BloomFilter bloom;
+	Konnector::BloomFilter bloom;
 	parseGlobalOpts(argc, argv);
 	// Arg parser to get `m' option in case set
 	for (int c; (c = getopt_long(argc, argv,
@@ -929,17 +1071,20 @@ int memberOf(int argc, char ** argv){
 /**
  * Calculate number of bases to trim from left end of sequence.
  */
-int calcLeftTrim(const Sequence& seq, unsigned k, const BloomFilter& bloom,
+int calcLeftTrim(const Sequence& seq, unsigned k, const Konnector::BloomFilter& bloom,
 	size_t minBranchLen)
 {
 	// Boost graph interface for Bloom filter
-	DBGBloom<BloomFilter> g(bloom);
+	DBGBloom<Konnector::BloomFilter> g(bloom);
 
 	// if this is the first k-mer we have found in
 	// Bloom filter, starting from the left end
 	// of the sequence
 	bool firstKmerMatch = true;
 
+	// longest branch of Bloom filter false positives
+	const unsigned fpTrim = 5;
+
 	KmerIterator it(seq, k);
 	for (; it != KmerIterator::end(); ++it) {
 
@@ -950,21 +1095,26 @@ int calcLeftTrim(const Sequence& seq, unsigned k, const BloomFilter& bloom,
 		if (!bloom[kmer])
 			continue;
 
-		// in degree, disregarding false branches
-		unsigned inDegree = trueBranches(kmer, REVERSE, g,
-				minBranchLen).size();
-		// out degree, disregarding false branches
-		unsigned outDegree = trueBranches(kmer, FORWARD, g,
-				minBranchLen).size();
+		PathExtensionResultCode leftResult, rightResult;
+		boost::tie(boost::tuples::ignore, leftResult)
+			= successor(kmer, REVERSE, g, minBranchLen, fpTrim);
+		boost::tie(boost::tuples::ignore, rightResult)
+			= successor(kmer, FORWARD, g, minBranchLen, fpTrim);
 
 		if (firstKmerMatch) {
-			bool leftTip = (inDegree == 0 && outDegree == 1);
-			bool rightTip = (inDegree == 1 && outDegree == 0);
+			bool leftTip = leftResult == ER_DEAD_END
+				&& rightResult == ER_LENGTH_LIMIT;
+			bool rightTip = leftResult == ER_LENGTH_LIMIT
+				&& rightResult == ER_DEAD_END;
 			if (!leftTip && !rightTip)
 				break;
-		} else if (inDegree != 1 || outDegree != 1) {
-			// end of linear path
-			break;
+		} else {
+			bool leftFork = leftResult == ER_AMBI_IN
+				|| leftResult == ER_AMBI_OUT;
+			bool rightFork = rightResult == ER_AMBI_IN
+				|| rightResult == ER_AMBI_OUT;
+			if (leftFork || rightFork)
+				break;
 		}
 
 		firstKmerMatch = false;
@@ -1000,7 +1150,7 @@ int trim(int argc, char** argv)
 		cerr << "Loading bloom filter from `"
 			<< bloomPath << "'...\n";
 
-	BloomFilter bloom;
+	Konnector::BloomFilter bloom;
 	istream *in = openInputStream(bloomPath);
 	assert_good(*in, bloomPath);
 	bloom.read(*in);
@@ -1103,6 +1253,9 @@ int main(int argc, char** argv)
 	else if (command == "compare") {
 		return compare(argc, argv);
 	}
+	else if (command == "graph") {
+		return graph(argc, argv);
+	}
 	else if (command == "kmers" || command == "getKmers") {
 		return memberOf(argc, argv);
 	}
