@@ -68,7 +68,7 @@ static const char USAGE_MESSAGE[] =
 "                               for FASTQ and SAM files [default]\n"
 "      --illumina-quality       zero quality is `@' (64), typically\n"
 "                               for qseq and export files\n"
-"  -t, --trim-length            max branch length to trim, in k-mers [k]\n"
+"  -t, --trim-length=N          max branch length to trim, in k-mers [k]\n"
 "  -v, --verbose                display verbose output\n"
 "      --version                output version information and exit\n"
 "\n"
@@ -90,10 +90,11 @@ static const char USAGE_MESSAGE[] =
 "  -C, --cov-track=FILE       WIG track with 0/1 indicating k-mers with\n"
 "                             coverage above the -c threshold. A reference\n"
 "                             must also be specified with -R.\n"
-"  -T, --trace-file=FILE      write debugging info about extension of\n"
-"                             each read to FILE\n"
+"  --read-log=FILE            write outcome of processing each read to FILE\n"
 "  -R, --ref=FILE             specify a reference genome. FILE may be\n"
-"                             FASTA, FASTQ, SAM, or BAM and may be gzipped."
+"                             FASTA, FASTQ, SAM, or BAM and may be gzipped.\n"
+"  -T, --trace-file=FILE      write debugging info about generation of\n"
+"                             each read to FILE\n"
 "\n"
 "Experimental Options:\n"
 "\n"
@@ -123,7 +124,7 @@ static const char shortopts[] = "b:C:g:H:i:j:k:K:o:q:Q:R:s:t:T:v";
 
 enum {
 	OPT_HELP = 1, OPT_VERSION, QR_SEED, MIN_KMER_COV,
-	CHECKPOINT, KEEP_CHECKPOINT, CHECKPOINT_PREFIX
+	CHECKPOINT, KEEP_CHECKPOINT, CHECKPOINT_PREFIX, READ_LOG,
 };
 
 static const struct option longopts[] = {
@@ -151,10 +152,11 @@ static const struct option longopts[] = {
 	{ "standard-quality",  no_argument, &opt::qualityOffset, 33 },
 	{ "illumina-quality",  no_argument, &opt::qualityOffset, 64 },
 	{ "qr-seed",           required_argument, NULL, QR_SEED },
+	{ "read-log",	       required_argument, NULL, READ_LOG },
 	{ "ref",               required_argument, NULL, 'R' },
-	{ "spaced-seed",       no_argument, NULL, 's' },
-	{ "trim-length",       no_argument, NULL, 't' },
-	{ "trace-file",        no_argument, NULL, 'T'},
+	{ "spaced-seed",       required_argument, NULL, 's' },
+	{ "trim-length",       required_argument, NULL, 't' },
+	{ "trace-file",        required_argument, NULL, 'T'},
 	{ "verbose",           no_argument, NULL, 'v' },
 	{ "version",           no_argument, NULL, OPT_VERSION },
 	{ NULL, 0, NULL, 0 }
@@ -228,7 +230,7 @@ void resumeAssemblyFromCheckpoint(int argc, char** argv,
 	HashAgnosticCascadingBloom solidKmerSet;
 
 	/* empty visited k-mers Bloom filter */
-	BTL::BloomFilter visitedKmerSet;
+	BloomFilter visitedKmerSet;
 
 	/* counters for progress messages */
 	BloomDBG::AssemblyCounters counters;
@@ -251,13 +253,22 @@ void resumeAssemblyFromCheckpoint(int argc, char** argv,
 	if (!params.tracePath.empty()) {
 		traceOut.open(params.tracePath.c_str());
 		assert_good(traceOut, params.tracePath);
-		BloomDBG::SeqExtensionResult::printHeaders(traceOut);
+		BloomDBG::ContigRecord::printHeaders(traceOut);
 		assert_good(traceOut, params.tracePath);
 	}
 
+	/* logs outcome of processing of each read (`--read-log`) */
+	std::ofstream readLogOut;
+	if (!params.readLogPath.empty()) {
+		readLogOut.open(params.readLogPath.c_str());
+		assert_good(readLogOut, params.readLogPath);
+		BloomDBG::ReadRecord::printHeaders(readLogOut);
+		assert_good(readLogOut, params.readLogPath);
+	}
+
 	/* bundle input/output streams for assembly */
 	BloomDBG::AssemblyStreams<FastaConcat>
-		streams(in, out, checkpointOut, traceOut);
+		streams(in, out, checkpointOut, traceOut, readLogOut);
 
 	/* restore state of Bloom filters, counters, and input/output streams */
 	BloomDBG::resumeFromCheckpoint(solidKmerSet, visitedKmerSet,
@@ -438,6 +449,9 @@ int main(int argc, char** argv)
 		  case CHECKPOINT_PREFIX:
 			arg >> params.checkpointPathPrefix;
 			break;
+		  case READ_LOG:
+			arg >> params.readLogPath;
+			break;
 		}
 
 		if (optarg != NULL && (!arg.eof() || arg.fail())) {
