@@ -1,9 +1,12 @@
 #ifndef BLOOM_DBG_H
 #define BLOOM_DBG_H 1
 
+#include "config.h"
+
 #include "BloomDBG/AssemblyParams.h"
 #include "BloomDBG/AssemblyCounters.h"
 #include "BloomDBG/RollingHashIterator.h"
+#include "Common/Hash.h"
 #include "Common/Uncompress.h"
 #include "Common/IOUtil.h"
 #include "Common/Sequence.h"
@@ -20,6 +23,7 @@
 #include "DataLayer/FastaConcat.h"
 #include "lib/bloomfilter/BloomFilter.hpp"
 
+#include <cmath>
 #include <string>
 #include <iostream>
 #include <sstream>
@@ -31,6 +35,20 @@
 # include <omp.h>
 #endif
 
+#if HAVE_GOOGLE_SPARSE_HASH_MAP
+
+#include <google/sparse_hash_set>
+typedef google::sparse_hash_set<RollingBloomDBGVertex,
+	hash<RollingBloomDBGVertex> > KmerHash;
+
+#else
+
+#include "Common/UnorderedSet.h"
+typedef unordered_set<RollingBloomDBGVertex,
+	hash<RollingBloomDBGVertex> > KmerHash;
+
+#endif
+
 namespace BloomDBG {
 
 	/**
@@ -125,193 +143,199 @@ namespace BloomDBG {
 		return seq;
 	}
 
+	enum SeedType { ST_BRANCH_KMER=0, ST_READ };
+
+	static inline std::string seedTypeStr(const SeedType& type)
+	{
+		switch(type) {
+		case ST_BRANCH_KMER:
+			return "BRANCH_KMER";
+		case ST_READ:
+			return "READ";
+		default:
+			break;
+		}
+		assert(false);
+	}
+
 	/**
 	 * Results for the extension of a read segment.
 	 * Each instance represents a row in the trace file generated
 	 * by the '-T' option for abyss-bloom-dbg.
 	 */
-	struct SeqExtensionResult
+	struct ContigRecord
 	{
-		/** FASTA ID for origin read */
-		std::string readId;
-		/**
-		 * Index of this segment within the read. (Prior to extension,
-		 * each read is split into segments at branching k-mers.)
-		 */
-		unsigned readSegmentId;
-		/** Total number of segments after splitting the read */
-		unsigned numReadSegments;
-		/** True if leftwards sequence extension was attempted */
-		bool extendedLeft;
-		/** True if rightwards sequence extension was attempted */
-		bool extendedRight;
+		/** output FASTA ID for this contig */
+		size_t contigID;
+		/** length of contig (bp) */
+		unsigned length;
+		/** FASTA ID of seeding read */
+		std::string readID;
+		/** Type of sequence used to seed contig (branch k-mer or full read) */
+		SeedType seedType;
+		/** Starting sequence prior to extensions */
+		Sequence seed;
 		/** Result code for attempted left sequence extension (e.g. DEAD END) */
 		PathExtensionResult leftExtensionResult;
 		/** Result code for attempted left sequence extension (e.g. DEAD END) */
 		PathExtensionResult rightExtensionResult;
-		/** Original length of the read segment prior to extension */
-		unsigned origLength;
-		/** length of left extension (bp) */
-		unsigned leftExtensionLength;
-		/** length of right extension (bp) */
-		unsigned rightExtensionLength;
-		/** total length of extended sequence (bp) */
-		unsigned extendedLength;
+
 		/**
 		 * True if the extended sequence was excluded from the output contigs
 		 * because it was redundant. (An identical sequence was generated
 		 * when extending a previous read.)
 		 */
-		bool redundantContig;
-		/** Contig ID assigned to extended segment */
-		size_t contigID;
+		bool redundant;
 
-		SeqExtensionResult() :
-			readId(),
-			readSegmentId(std::numeric_limits<unsigned>::max()),
-			numReadSegments(std::numeric_limits<unsigned>::max()),
-			extendedLeft(false),
-			extendedRight(false),
-			leftExtensionResult(DEAD_END),
-			rightExtensionResult(DEAD_END),
-			origLength(std::numeric_limits<unsigned>::max()),
-			leftExtensionLength(std::numeric_limits<unsigned>::max()),
-			rightExtensionLength(std::numeric_limits<unsigned>::max()),
-			extendedLength(std::numeric_limits<unsigned>::max()),
-			redundantContig(false),
-			contigID(std::numeric_limits<size_t>::max()) {}
-
-		bool initialized() const
-		{
-			return !readId.empty() &&
-				readSegmentId != std::numeric_limits<unsigned>::max() &&
-				numReadSegments != std::numeric_limits<unsigned>::max() &&
-				origLength != std::numeric_limits<unsigned>::max() &&
-				leftExtensionLength != std::numeric_limits<unsigned>::max() &&
-				rightExtensionLength != std::numeric_limits<unsigned>::max() &&
-				extendedLength != std::numeric_limits<unsigned>::max();
-		}
+		ContigRecord() :
+			contigID(std::numeric_limits<size_t>::max()),
+			readID(),
+			leftExtensionResult(std::make_pair(0, ER_DEAD_END)),
+			rightExtensionResult(std::make_pair(0, ER_DEAD_END)),
+			redundant(false)
+			{}
 
 		static std::ostream& printHeaders(std::ostream& out)
 		{
-			out << "read_id\t"
-				<< "read_segment_id\t"
-				<< "num_read_segments\t"
-				<< "left_extension_result\t"
-				<< "right_extension_result\t"
-				<< "orig_length\t"
-				<< "left_extension_len\t"
-				<< "right_extension_len\t"
-				<< "extended_length\t"
-				<< "redundant_contig\t"
-				<< "contig_id\n";
+			out << "contig_id" << '\t'
+				<< "length" << '\t'
+				<< "redundant" << '\t'
+				<< "read_id" << '\t'
+				<< "left_result" << '\t'
+				<< "left_extension" << '\t'
+				<< "right_result" << '\t'
+				<< "right_extension" << '\t'
+				<< "seed_type" << '\t'
+				<< "seed_length" << '\t'
+				<< "seed" << '\n';
 			return out;
 		}
 
 		friend std::ostream& operator <<(std::ostream& out,
-			const SeqExtensionResult& o)
+			const ContigRecord& o)
 		{
-			if (o.redundantContig) {
-				out << o.readId << '\t'
-					<< o.readSegmentId << '\t'
-					<< o.numReadSegments << '\t'
-					<< "-\t"
-					<< "-\t"
-					<< o.origLength << '\t'
-					<< "-\t"
-					<< "-\t"
-					<< "-\t"
-					<< "true" << '\t'
-					<< "-\n";
-			} else {
-				out << o.readId << '\t'
-					<< o.readSegmentId << '\t'
-					<< o.numReadSegments << '\t';
-				if (o.extendedLeft)
-					out << pathExtensionResultStr(o.leftExtensionResult) << '\t';
-				else
-					out << "-\t";
-				if (o.extendedRight)
-					out << pathExtensionResultStr(o.rightExtensionResult) << '\t';
-				else
-					out << "-\t";
-				out << o.origLength << '\t';
-				if (o.extendedLeft)
-					out << o.leftExtensionLength << '\t';
-				else
-					out << "-\t";
-				if (o.extendedRight)
-					out << o.rightExtensionLength << '\t';
-				else
-					out << "-\t";
-				out << o.extendedLength << '\t'
-					<< "false" << '\t'
-					<< o.contigID << '\n';
-			}
+			if (o.redundant)
+				out << "NA" << '\t';
+			else
+				out << o.contigID << '\t';
+
+			out << o.length << '\t'
+				<< o.redundant << '\t'
+				<< o.readID << '\t';
+
+			if (o.leftExtensionResult.first > 0)
+				out << pathExtensionResultStr(o.leftExtensionResult.second) << '\t'
+					<< o.leftExtensionResult.first << '\t';
+			else
+				out << "NA" << '\t' << "NA" << '\t';
+
+			if (o.rightExtensionResult.first > 0)
+				out << pathExtensionResultStr(o.rightExtensionResult.second) << '\t'
+					<< o.rightExtensionResult.first << '\t';
+			else
+				out << "NA" << '\t' << "NA" << '\t';
+
+			out << seedTypeStr(o.seedType) << '\t'
+				<< o.seed.length() << '\t'
+				<< o.seed << '\n';
+
 			return out;
 		}
 	};
 
-	/**
-	 * Extend a sequence left (REVERSE) or right (FORWARD) within the de Bruijn
-	 * graph until either a branching point or a dead-end is encountered.
-	 */
-	template <typename GraphT>
-	inline static PathExtensionResult extendSeq(Sequence& seq, Direction dir,
-		unsigned k, unsigned numHashes, unsigned minBranchLen, const GraphT& graph)
+	enum ReadResult {
+		RR_UNINITIALIZED=0,
+		RR_SHORTER_THAN_K,
+		RR_NON_ACGT,
+		RR_BLUNT_END,
+		RR_NOT_SOLID,
+		RR_ALL_KMERS_VISITED,
+		RR_ALL_BRANCH_KMERS_VISITED,
+		RR_GENERATED_CONTIGS
+	};
+
+	static inline std::string readResultStr(const ReadResult& result)
 	{
-		assert(seq.length() >= k);
+		switch(result) {
+		case RR_UNINITIALIZED:
+			return "NA";
+		case RR_SHORTER_THAN_K:
+			return "SHORTER_THAN_K";
+		case RR_NON_ACGT:
+			return "NON_ACGT";
+		case RR_BLUNT_END:
+			return "BLUNT_END";
+		case RR_NOT_SOLID:
+			return "NOT_SOLID";
+		case RR_ALL_KMERS_VISITED:
+			return "ALL_KMERS_VISITED";
+		case RR_ALL_BRANCH_KMERS_VISITED:
+			return "ALL_BRANCH_KMERS_VISITED";
+		case RR_GENERATED_CONTIGS:
+			return "GENERATED_CONTIGS";
+		default:
+			break;
+		}
+		assert(false);
+	}
 
-		/* Convert sequence to path in DBG */
-		Path<Vertex> path = seqToPath(seq, k, numHashes);
+	/**
+	 * Records results of processing a sequencing read.
+	 * Each instance represents a row in the read log file generated
+	 * by the `--read-log` option of `abyss-bloom-dbg`.
+	 */
+	struct ReadRecord
+	{
+		/** FASTA ID for this read */
+		std::string readID;
+		/** outcome of processing the read */
+		ReadResult result;
 
-		/* Extend path */
-		ExtendPathParams params;
-		params.trimLen = minBranchLen - 1;
-		params.maxLen = NO_LIMIT;
-		PathExtensionResult result =
-			extendPath(path, dir, graph, params);
+		ReadRecord() : readID(), result(RR_UNINITIALIZED) {}
 
-		/* Convert extended path back to sequence */
-		Sequence extendedSeq = pathToSeq(path, k);
+		ReadRecord(const std::string& readID)
+			: readID(readID), result(RR_UNINITIALIZED) {}
 
-		/*
-		 * If a spaced seed is in effect, short paths may result in
-		 * sequences containing 'N's.  However, since we only extend
-		 * "perfect reads", we can replace the 'N's with the correct
-		 * bases by overlaying the seed sequence.
-		 */
-		if (dir == FORWARD) {
-			overlaySeq(seq, extendedSeq, 0);
-		} else {
-			assert(dir == REVERSE);
-			overlaySeq(seq, extendedSeq, extendedSeq.length() - seq.length());
+		ReadRecord(const std::string& readID, ReadResult result)
+			: readID(readID), result(result) {}
+
+		static std::ostream& printHeaders(std::ostream& out)
+		{
+			out << "read_id" << '\t'
+				<< "result" << '\n';
+			return out;
 		}
 
-		/*
-		 * Replace orig seq with extended version.
-		 */
-		seq = extendedSeq;
+		friend std::ostream& operator <<(std::ostream& out,
+			const ReadRecord& o)
+		{
+			out << o.readID << '\t'
+				<< readResultStr(o.result) << '\n';
 
-		/* Return true if sequence was successfully extended */
-		return result;
+			return out;
+		}
+	};
+
+	/** Print an intermediate progress message during assembly */
+	void readsProgressMessage(AssemblyCounters counters)
+	{
+		std::cerr
+			<< "Processed " << counters.readsProcessed << " reads"
+			<< ", solid reads: " << counters.solidReads
+			<< " (" << std::setprecision(3) << (float)100
+			* counters.solidReads / counters.readsProcessed << "%)"
+			<< ", visited reads: " << counters.visitedReads
+			<< " (" << std::setprecision(3) << (float)100
+			* counters.visitedReads / counters.readsProcessed << "%)"
+			<< std::endl;
 	}
 
 	/** Print an intermediate progress message during assembly */
-	void printProgressMessage(AssemblyCounters counters)
+	void basesProgressMessage(AssemblyCounters counters)
 	{
-#pragma omp critical(cerr)
-		std::cerr
-			<< "Extended " << counters.readsExtended
-			<< " of " << counters.readsProcessed
-			<< " reads (" << std::setprecision(3) << (float)100
-			* counters.readsExtended / counters.readsProcessed
-			<< "%), saw " << counters.solidReads
-			<< " of " << counters.readsProcessed
-			<< " solid reads (" << std::setprecision(3) << (float)100
-		    * counters.solidReads / counters.readsProcessed
-			<< "%), assembled " << counters.basesAssembled
-			<< " bp so far" << std::endl;
+		std::cerr << "Assembled " << counters.basesAssembled << " bp in "
+			<< counters.contigID + 1 << " contigs"
+			<< std::endl;
 	}
 
 	/**
@@ -351,68 +375,6 @@ namespace BloomDBG {
 		return splitPaths;
 	}
 
-	/**
-	 * Split a sequence at branching k-mers (degree > 2).
-	 * Branching k-mers are shared between the resulting sequence
-	 * segments.
-	 */
-	template <typename GraphT>
-	inline static std::vector<Sequence>
-	splitSeq(const Sequence& seq, unsigned k, unsigned numHashes,
-		const GraphT& dbg, unsigned minBranchLen)
-	{
-		assert(seq.length() >= k);
-
-		typedef typename boost::graph_traits<GraphT>::vertex_descriptor V;
-		typedef typename Path<V>::const_iterator PathIt;
-
-		std::vector<Sequence> segments;
-		Path<V> path = seqToPath(seq, k, numHashes);
-		PathIt start = path.begin();
-		PathIt end = path.begin();
-
-		for (; end != path.end(); ++end) {
-			std::vector<V> inBranches = trueBranches(*end, REVERSE, dbg,
-				minBranchLen-1);
-			unsigned inDegree = inBranches.size();
-			/*
-			 * Tricky: Include the read itself in the list of valid
-			 * incoming branches, even if it is shorter than trimLen.
-			 */
-			if (end > path.begin() && std::find(inBranches.begin(),
-				inBranches.end(), *(end - 1)) == inBranches.end()) {
-				inDegree++;
-			}
-			std::vector<V> outBranches = trueBranches(*end, FORWARD, dbg,
-				minBranchLen-1);
-			unsigned outDegree = outBranches.size();
-			/*
-			 * Tricky: Include the read itself in the list of valid
-			 * outgoing branches, even if it is shorter than trimLen.
-			 */
-			if (end < path.end() - 1 && std::find(outBranches.begin(),
-				outBranches.end(), *(end + 1)) == outBranches.end()) {
-				outDegree++;
-			}
-			if (inDegree > 1 || outDegree > 1) {
-				/* we've hit a branching point -- end the current
-				 * segment and start a new one */
-				Sequence segment = seq.substr(start - path.begin(),
-					end - start + k);
-				segments.push_back(segment);
-				start = end;
-			}
-		}
-		if (segments.empty() || segments.back().length() > k) {
-			Sequence segment = seq.substr(start - path.begin(),
-				end - start + k);
-			segments.push_back(segment);
-		}
-
-		assert(segments.size() >= 1);
-		return segments;
-	}
-
 	/**
 	 * Trim a sequence down to the longest contiguous subsequence
 	 * of "good" k-mers.  If the sequence has length < k or contains
@@ -474,54 +436,6 @@ namespace BloomDBG {
 		seq = seq.substr(maxMatchStart, maxMatchLen + k - 1);
 	}
 
-	/**
-	 * Ensure that branching k-mers are not repeated in the output
-	 * contigs by selectively trimming contig ends.
-	 *
-	 * The idea is to keep a branch k-mer if the edge leading to it
-	 * is unambigous. For example, in the diagram below the contig
-	 * generated from the right side would include the branching k-mer
-	 * k5, whereas the two contigs entering on the left would discard it:
-	 *
-	 * ...-k1-k2
-	 *          \
-	 *           k5-k6-...
-	 *          /
-	 * ...-k3-k4
-	 *
-	 * @param seq the contig to trim
-	 * @param k k-mer size
-	 * @param numHashes number of Bloom filter hash functions
-	 * @param minBranchLen minimum length of a "true" branch (shorter
-	 * branches are assumed to be caused by sequencing errors or
-	 * Bloom filter false positives).
-	 */
-	template <typename GraphT>
-		inline static void trimBranchKmers(Sequence& seq,
-			unsigned k, unsigned numHashes, unsigned minBranchLen,
-			const GraphT& dbg)
-	{
-		assert(seq.length() >= k);
-
-		if (seq.length() == k)
-			return;
-
-		Sequence firstKmer = seq.substr(0, k);
-		Vertex vFirst(firstKmer.c_str(), RollingHash(firstKmer, numHashes, k));
-		unsigned outDegree = trueDegree(vFirst, FORWARD, dbg, minBranchLen - 1);
-		if (outDegree > 1)
-			seq.erase(0, 1);
-
-		if (seq.length() == k)
-			return;
-
-		Sequence lastKmer = seq.substr(seq.length()-k);
-		Vertex vLast(lastKmer.c_str(), RollingHash(lastKmer, numHashes, k));
-		unsigned inDegree = trueDegree(vLast, REVERSE, dbg, minBranchLen - 1);
-		if (inDegree > 1)
-			seq.erase(seq.length()-1, 1);
-	}
-
 	/**
 	 * Append a contig to the output FASTA stream.
 	 */
@@ -544,182 +458,14 @@ namespace BloomDBG {
 		contig.id = id.str();
 		contig.comment = comment.str();
 
-		/* set seq (in canonical orientation) */
-		Sequence rcSeq = reverseComplement(seq);
-		contig.seq = (seq < rcSeq) ? seq : rcSeq;
+		/* set seq */
+		contig.seq = seq;
 
 		/* output FASTQ record */
 		out << contig;
 		assert(out);
 	}
 
-	/**
-	 * Trim contiguous stretches of previously-assembled k-mers from
-	 * both ends of a contig.
-	 *
-	 * @param seq contig to be trimmed
-	 * @param assembledKmerSet Bloom filter of k-mers from previously
-	 * assembled contigs
-	 */
-	template <typename BloomT>
-	inline static void trimContigOverlaps(Sequence &seq,
-		const BloomT& assembledKmerSet)
-	{
-		const unsigned k = assembledKmerSet.getKmerSize();
-		const unsigned numHashes = assembledKmerSet.getHashNum();
-
-		/* trim previously assembled k-mers from start of sequence */
-		RollingHashIterator fwd(seq, numHashes, k);
-		for (; fwd != RollingHashIterator::end(); ++fwd) {
-			if (!assembledKmerSet.contains(*fwd))
-				break;
-		}
-		if (fwd.pos() > 0)
-			seq.erase(0, fwd.pos());
-
-		/* trim previously assembled k-mers from end of sequence */
-		Sequence rcSeq = reverseComplement(seq);
-		RollingHashIterator rev(rcSeq, numHashes, k);
-		for (; rev != RollingHashIterator::end(); ++rev) {
-			if (!assembledKmerSet.contains(*rev))
-				break;
-		}
-		if (rev.pos() > 0)
-			rcSeq.erase(0, rev.pos());
-
-		/* flip seq back to original orientation */
-		seq = reverseComplement(rcSeq);
-
-		assert(seq.length() >= k);
-	}
-
-	/**
-	 * Split a read at branching points in the de Bruijn graph and
-	 * then extend each segment left and right, up to the next
-	 * branching point or dead end.
-	 *
-	 * @param read read to be assembled
-	 * @param dbg Boost graph interface to de Bruijn graph
-	 * @param assembledKmerSet Bloom filter containing k-mers of
-	 * previously assembled contigs
-	 * @param params command line options for the assembly
-	 * (e.g. k-mer coverage threshold)
-	 * @param counters counter variables used for generating assembly
-	 * progress messages.
-	 * @param streams input/output streams used during assembly
-	 */
-	template <typename GraphT, typename BloomT, typename AssemblyStreamsT>
-	inline static void extendRead(const FastaRecord& read,
-		const GraphT& dbg, BloomT& assembledKmerSet,
-		const AssemblyParams& params, AssemblyCounters& counters,
-		AssemblyStreamsT& streams)
-	{
-		const unsigned k = params.k;
-		const unsigned numHashes = params.numHashes;
-		const unsigned minBranchLen = params.trim + 1;
-
-		std::ostream& out = streams.out;
-		std::ostream& checkpointOut = streams.checkpointOut;
-		std::ostream& traceOut = streams.traceOut;
-
-		if (params.verbose >= 2) {
-#pragma omp critical(cerr)
-			std::cerr << "Extending read: " << read.id << std::endl;
-		}
-
-		/* split read at branching points (prevents over-assembly) */
-		std::vector<Sequence> segments = splitSeq(read.seq, k,
-			numHashes, dbg, minBranchLen);
-
-		for (std::vector<Sequence>::iterator it = segments.begin();
-			 it != segments.end(); ++it) {
-
-			Sequence& seq = *it;
-
-			/*
-			 * track results of sequence extension attempt for
-			 * trace file ('-T' option).
-			 */
-			SeqExtensionResult traceResult;
-			traceResult.readId = read.id;
-			traceResult.readSegmentId = it - segments.begin() + 1;
-			traceResult.numReadSegments = segments.size();
-			traceResult.origLength = seq.length();
-			traceResult.leftExtensionLength = 0;
-			traceResult.rightExtensionLength = 0;
-			traceResult.redundantContig = true;
-
-			/*
-			 * extend first and last segments only, since
-			 * internal segments are bounded by branching
-			 * points.
-			 */
-			if (it == segments.begin()) {
-				traceResult.extendedLeft = true;
-				traceResult.leftExtensionResult = extendSeq(seq,
-					REVERSE, k, numHashes, minBranchLen, dbg);
-				traceResult.leftExtensionLength =
-					seq.length() - traceResult.origLength;
-			}
-			if (it == segments.end() - 1) {
-				unsigned origLength = seq.length();
-				traceResult.extendedRight = true;
-				traceResult.rightExtensionResult = extendSeq(seq,
-					FORWARD, k, numHashes, minBranchLen, dbg);
-				traceResult.rightExtensionLength =
-					seq.length() - origLength;
-			}
-			traceResult.extendedLength = seq.length();
-
-			/* ensure branching k-mers are included only once in output */
-			trimBranchKmers(seq, k, numHashes, minBranchLen, dbg);
-
-			/*
-			 * check assembledKmerSet again to prevent race
-			 * condition. (Otherwise, the same contig may be
-			 * generated multiple times.)
-			 */
-#pragma omp critical(assembledKmerSet)
-			if (!allKmersInBloom(seq, assembledKmerSet)) {
-
-				/* trim previously assembled k-mers from both ends */
-				trimContigOverlaps(seq, assembledKmerSet);
-
-				/* mark remaining k-mers as assembled */
-				addKmersToBloom(seq, assembledKmerSet);
-
-				/* add contig to output FASTA */
-				printContig(seq, counters.contigID, read.id, k, out);
-
-				/* add contig to checkpoint FASTA file */
-				if (params.checkpointsEnabled())
-					printContig(seq, counters.contigID, read.id, k,
-						checkpointOut);
-
-				/* update counters / trace results */
-				traceResult.redundantContig = false;
-				traceResult.contigID = counters.contigID;
-				counters.basesAssembled += seq.length();
-				counters.contigID++;
-			}
-
-			/* trace file output ('-T' option) */
-#pragma omp critical(traceOut)
-			if (!params.tracePath.empty()) {
-				assert(traceResult.initialized());
-				traceOut << traceResult;
-				assert_good(traceOut, params.tracePath);
-			}
-
-		}  /* for each read segment */
-
-		if (params.verbose >= 2) {
-#pragma omp critical(cerr)
-			std::cerr << "Finished extending read: " << read.id << std::endl;
-		}
-
-	}
-
 	/**
 	 * Return true if the left end of the given sequence is a blunt end
 	 * in the Bloom filterde Bruijn graph. PRECONDITION: `seq` does not contain
@@ -765,51 +511,350 @@ namespace BloomDBG {
 		return false;
 	}
 
+	/**
+	 * Output a contig sequence if it is not redundant, i.e. it has not already
+	 * been generated from a different read / thread of execution.
+	 */
+	template <typename AssembledKmerSetT, typename AssemblyStreamsT>
+	inline static void outputContig(const Path<Vertex>& contigPath,
+		ContigRecord& rec, AssembledKmerSetT& assembledKmerSet,
+		KmerHash& contigEndKmers, const AssemblyParams& params,
+		AssemblyCounters& counters, AssemblyStreamsT& streams)
+	{
+		const unsigned fpLookAhead = 5;
+
+		Sequence seq = pathToSeq(contigPath, params.k);
+
+		/* vertices representing start/end k-mers of contig */
+
+		Sequence kmer1 = seq.substr(0, params.k);
+		canonicalize(kmer1);
+		RollingHash hash1(kmer1.c_str(), params.numHashes, params.k);
+		Vertex v1(kmer1.c_str(), hash1);
+
+		Sequence kmer2 = seq.substr(seq.length() - params.k);
+		canonicalize(kmer2);
+		RollingHash hash2(kmer2.c_str(), params.numHashes, params.k);
+		Vertex v2(kmer2.c_str(), hash2);
+
+		bool redundant = false;
+#pragma omp critical(redundancyCheck)
+		{
+			/*
+			 * If we use `assembledKmerSet` to check very short contigs,
+			 * we may get full-length matches purely due to Bloom filter
+			 * positives.  For such contigs, we additionally track
+			 * the start and end in a separate hash table called
+			 * `contigEndKmers`.
+			 */
+			if (seq.length() < params.k + fpLookAhead - 1) {
+
+				if (contigEndKmers.find(v1) != contigEndKmers.end()
+					&& contigEndKmers.find(v2) != contigEndKmers.end()) {
+					redundant = true;
+				} else {
+					contigEndKmers.insert(v1);
+					contigEndKmers.insert(v2);
+				}
+
+			} else if (allKmersInBloom(seq, assembledKmerSet)) {
+				redundant = true;
+			}
+
+			if (!redundant) {
+
+				/* mark remaining k-mers as assembled */
+				addKmersToBloom(seq, assembledKmerSet);
+
+			}
+		}
+		rec.redundant = redundant;
+
+		if (!redundant)
+		{
+#pragma omp critical(fasta)
+			{
+				/* add contig to output FASTA */
+				printContig(seq, counters.contigID, rec.readID, params.k,
+					streams.out);
+
+				/* add contig to checkpoint FASTA file */
+				if (params.checkpointsEnabled())
+					printContig(seq, counters.contigID, rec.readID, params.k,
+						streams.checkpointOut);
+
+				rec.contigID = counters.contigID;
+				rec.length = seq.length();
+
+				counters.contigID++;
+				counters.basesAssembled += seq.length();
+			}
+		}
+
+#pragma omp critical(trace)
+		streams.traceOut << rec;
+
+	}
+
+	enum ContigType { CT_LINEAR, CT_CIRCULAR, CT_HAIRPIN };
+
+	template <typename GraphT>
+	static inline ContigType getContigType(const Path<Vertex>& contigPath,
+		const GraphT& dbg)
+	{
+		if (edge(contigPath.back(), contigPath.front(), dbg).second) {
+
+			Vertex v = contigPath.front().clone();
+			v.shift(ANTISENSE, contigPath.back().kmer().getBase(0));
+
+			if (v.kmer() == contigPath.back().kmer())
+				return CT_CIRCULAR;
+			else
+				return CT_HAIRPIN;
+
+		}
+
+		return CT_LINEAR;
+	}
+
+	/** Special case behaviour for circular/hairpin contigs */
+	template <typename GraphT>
+	static inline void preprocessCircularContig(Path<Vertex>& contigPath,
+		const GraphT& dbg, unsigned trim)
+	{
+		assert(!contigPath.empty());
+
+		ContigType contigType = getContigType(contigPath, dbg);
+		assert(contigType != CT_LINEAR);
+
+		if (contigPath.size() <= 2)
+			return;
+
+		/* longest branch of Bloom filter positives */
+		const unsigned fpTrim = 5;
+
+		/*
+		 * Note: A circular/hairpin contig contains at most two branch k-mers.
+		 * And if it does contain branch k-mers, they will be the first/last
+		 * k-mers.
+		 *
+		 * If only one end of the circular/hairpin contig is a branch k-mer,
+		 * add that k-mer to the other end of the contig as well. This
+		 * enables the usual branch k-mer trimming logic for linear contigs to
+		 * produce the correct results downstream.
+		 */
+
+		bool branchStart = ambiguous(contigPath.front(), FORWARD, dbg, trim, fpTrim)
+			|| ambiguous(contigPath.front(), REVERSE, dbg, trim, fpTrim);
+
+		bool branchEnd = ambiguous(contigPath.back(), FORWARD, dbg, trim, fpTrim)
+			|| ambiguous(contigPath.back(), REVERSE, dbg, trim, fpTrim);
+
+		if (branchStart && !branchEnd) {
+
+			if (contigType == CT_CIRCULAR) {
+				contigPath.push_back(contigPath.front());
+			} else {
+				assert(contigType == CT_HAIRPIN);
+				Vertex rc = contigPath.front().clone();
+				rc.reverseComplement();
+				contigPath.push_back(rc);
+			}
+
+		} else if (!branchStart && branchEnd) {
+
+			if (contigType == CT_CIRCULAR) {
+				contigPath.push_front(contigPath.back());
+			} else {
+				assert(contigType == CT_HAIRPIN);
+				Vertex rc = contigPath.back().clone();
+				rc.reverseComplement();
+				contigPath.push_front(rc);
+			}
+
+		}
+
+	}
+
+	/**
+	 * Ensure that branching k-mers are not repeated in the output
+	 * contigs by selectively trimming contig ends.
+	 *
+	 * The idea is to keep a branch k-mer at the end of contig only
+	 * if the edge leading up to it is unambigous. For example, in the
+	 * diagram below the contig generated from the right side would
+	 * include the branching k-mer k5, whereas the two contigs entering
+	 * on the left would discard it:
+	 *
+	 * ...-k1-k2
+	 *          \
+	 *           k5-k6-...
+	 *          /
+	 * ...-k3-k4
+	 *
+	 * If a branch k-mer has both indegree > 1 and outdegree > 1, it is
+	 * output separately as its own contig of length k.
+	 */
+	template <typename GraphT>
+	inline static void trimBranchKmers(
+		Path<Vertex>& contigPath, const GraphT& dbg, unsigned trim)
+	{
+		assert(!contigPath.empty());
+
+		if (contigPath.size() == 1)
+			return;
+
+		/* special case: circular/hairpin contigs */
+
+		ContigType contigType = getContigType(contigPath, dbg);
+		if (contigType == CT_CIRCULAR || contigType == CT_HAIRPIN)
+			preprocessCircularContig(contigPath, dbg, trim);
+
+		unsigned l = contigPath.size();
+
+		/* longest branch of Bloom filter false positives */
+		const unsigned fpTrim = 5;
+
+		bool ambiguous1 = ambiguous(contigPath.at(0), contigPath.at(1),
+			FORWARD, dbg, trim, fpTrim);
+
+		bool ambiguous2 = ambiguous(contigPath.at(l-1), contigPath.at(l-2),
+			REVERSE, dbg, trim, fpTrim);
+
+		if (ambiguous1)
+			contigPath.pop_front();
+
+		assert(!contigPath.empty());
+
+		if (ambiguous2)
+			contigPath.pop_back();
+
+		assert(!contigPath.empty());
+	}
+
+	bool isTip(unsigned length, PathExtensionResultCode leftResult,
+		PathExtensionResultCode rightResult, unsigned trim)
+	{
+		if (length > trim)
+			return false;
+
+		if (leftResult == ER_DEAD_END &&
+			(rightResult == ER_DEAD_END || rightResult == ER_AMBI_IN))
+			return true;
+
+		if (rightResult == ER_DEAD_END &&
+			(leftResult == ER_DEAD_END || leftResult == ER_AMBI_IN))
+			return true;
+
+		return false;
+	}
+
 	/**
 	 * Decide if a read should be extended and if so extend it into a contig.
 	 */
 	template <typename SolidKmerSetT, typename AssembledKmerSetT,
 		typename AssemblyStreamsT>
-	static inline void processRead(const FastaRecord& rec,
+	static inline ReadRecord processRead(const FastaRecord& rec,
 		const SolidKmerSetT& solidKmerSet, AssembledKmerSetT& assembledKmerSet,
+		KmerHash& contigEndKmers, KmerHash& visitedBranchKmers,
 		const AssemblyParams& params, AssemblyCounters& counters,
 		AssemblyStreamsT& streams)
 	{
+		(void)visitedBranchKmers;
+
+		typedef typename Path<Vertex>::iterator PathIt;
+
 		/* Boost graph API for Bloom filter */
-		RollingBloomDBG<SolidKmerSetT> graph(solidKmerSet);
+		RollingBloomDBG<SolidKmerSetT> dbg(solidKmerSet);
 
 		unsigned k = params.k;
 		const Sequence& seq = rec.seq;
 
 		/* we can't extend reads shorter than k */
 		if (seq.length() < k)
-			return;
+			return ReadRecord(rec.id, RR_SHORTER_THAN_K);
 
 		/* skip reads with non-ACGT chars */
 		if (!allACGT(seq))
-			return;
+			return ReadRecord(rec.id, RR_NON_ACGT);
 
 		/* don't extend reads that are tips */
-		if (hasBluntEnd(seq, graph, params))
-			return;
+		if (hasBluntEnd(seq, dbg, params))
+			return ReadRecord(rec.id, RR_BLUNT_END);
 
 		/* only extend "solid" reads */
 		if (!allKmersInBloom(seq, solidKmerSet))
-			return;
+			return ReadRecord(rec.id, RR_NOT_SOLID);
 
 #pragma omp atomic
 		counters.solidReads++;
 
 		/* skip reads in previously assembled regions */
-		if (allKmersInBloom(seq, assembledKmerSet))
-			return;
-
-		/* extend the read into a contig */
-		extendRead(rec, graph, assembledKmerSet, params,
-			counters, streams);
-
+		if (allKmersInBloom(seq, assembledKmerSet)) {
 #pragma omp atomic
-		counters.readsExtended++;
+			counters.visitedReads++;
+			return ReadRecord(rec.id, RR_ALL_KMERS_VISITED);
+		}
+
+		/*
+		 * We use `assembledKmers` to track read k-mers
+		 * that have already been included in the output contigs, and
+		 * therefore do not need to be processed (extended) again.
+		 * Note that we do not use `assembledKmerSet` (a Bloom filter)
+		 * for this purpose because Bloom filter false positives
+		 * may cause us to omit short contigs (e.g. contigs with length k).
+		 */
+		unordered_set<Vertex> assembledKmers;
+
+		Path<Vertex> path = seqToPath(rec.seq, params.k, params.numHashes);
+		for (PathIt it = path.begin(); it != path.end(); ++it) {
+
+			if (assembledKmers.find(*it) != assembledKmers.end())
+				continue;
+
+			ExtendPathParams extendParams;
+			extendParams.trimLen = params.trim;
+			extendParams.fpTrim = 5;
+			extendParams.maxLen = NO_LIMIT;
+			extendParams.lookBehind = true;
+			extendParams.lookBehindStartVertex = false;
+
+			ContigRecord contigRec;
+			contigRec.readID = rec.id;
+			contigRec.seedType = ST_READ;
+			contigRec.seed = it->kmer().c_str();
+
+			Path<Vertex> contigPath;
+			contigPath.push_back(*it);
+
+			contigRec.leftExtensionResult
+				= extendPath(contigPath, REVERSE, dbg, extendParams);
+
+			contigRec.rightExtensionResult
+				= extendPath(contigPath, FORWARD, dbg, extendParams);
+
+			PathExtensionResultCode leftResult
+				= contigRec.leftExtensionResult.second;
+			PathExtensionResultCode rightResult
+				= contigRec.rightExtensionResult.second;
+
+			if (!isTip(contigPath.size(), leftResult, rightResult, params.trim))
+			{
+				/* selectively trim branch k-mers from contig ends */
+				trimBranchKmers(contigPath, dbg, params.trim);
+
+				/* output contig to FASTA file */
+				outputContig(contigPath, contigRec, assembledKmerSet,
+					contigEndKmers, params, counters, streams);
+			}
+
+			/* mark contig k-mers as visited */
+			for (PathIt it2 = contigPath.begin(); it2 != contigPath.end(); ++it2)
+				assembledKmers.insert(*it2);
+
+		}
+
+		return ReadRecord(rec.id, RR_GENERATED_CONTIGS);
 	}
 
 	/**
@@ -834,7 +879,7 @@ namespace BloomDBG {
 			std::ostream& out)
 	{
 		/* k-mers in previously assembled contigs */
-		BTL::BloomFilter visitedKmerSet(solidKmerSet.size(),
+		BloomFilter assembledKmerSet(solidKmerSet.size(),
 			solidKmerSet.getHashNum(), solidKmerSet.getKmerSize());
 
 		/* counters for progress messages */
@@ -858,15 +903,25 @@ namespace BloomDBG {
 		if (!params.tracePath.empty()) {
 			traceOut.open(params.tracePath.c_str());
 			assert_good(traceOut, params.tracePath);
-			SeqExtensionResult::printHeaders(traceOut);
+			ContigRecord::printHeaders(traceOut);
 			assert_good(traceOut, params.tracePath);
 		}
 
+		/* logs outcome of processing of each read (`--read-log`) */
+		std::ofstream readLogOut;
+		if (!params.readLogPath.empty()) {
+			readLogOut.open(params.readLogPath.c_str());
+			assert_good(readLogOut, params.readLogPath);
+			ReadRecord::printHeaders(readLogOut);
+			assert_good(readLogOut, params.readLogPath);
+		}
+
 		/* bundle output streams */
-		AssemblyStreams<FastaConcat> streams(in, out, checkpointOut, traceOut);
+		AssemblyStreams<FastaConcat> streams(in, out, checkpointOut,
+			traceOut, readLogOut);
 
 		/* run the assembly */
-		assemble(solidKmerSet, visitedKmerSet, counters, params, streams);
+		assemble(solidKmerSet, assembledKmerSet, counters, params, streams);
 	}
 
 	/**
@@ -881,17 +936,17 @@ namespace BloomDBG {
 	 * @param genomeSize approx genome size
 	 * @param goodKmerSet Bloom filter containing k-mers that
 	 * occur more than once in the input data
-	 * @param visitedKmerSet Bloom filter containing k-mers that have
+	 * @param assembledKmerSet Bloom filter containing k-mers that have
 	 * already been included in contigs
 	 * @param in input stream for sequencing reads (FASTA)
 	 * @param out output stream for contigs (FASTA)
 	 * @param verbose set to true to print progress messages to
 	 * STDERR
 	 */
-	template <typename SolidKmerSetT, typename VisitedKmerSetT,
+	template <typename SolidKmerSetT, typename AssembledKmerSetT,
 		typename InputReadStreamT>
 	inline static void assemble(const SolidKmerSetT& goodKmerSet,
-		VisitedKmerSetT& assembledKmerSet, AssemblyCounters& counters,
+		AssembledKmerSetT& assembledKmerSet, AssemblyCounters& counters,
 		const AssemblyParams& params,
 		AssemblyStreams<InputReadStreamT>& streams)
 	{
@@ -899,8 +954,6 @@ namespace BloomDBG {
 
 		/* per-thread I/O buffer (size is in bases) */
 		const size_t SEQ_BUFFER_SIZE = 1000000;
-		/* controls frequency of progress messages */
-		const unsigned PROGRESS_STEP = 1000;
 
 		if (params.verbose)
 			std::cerr << "Trimming branches " << params.trim
@@ -909,6 +962,23 @@ namespace BloomDBG {
 		InputReadStreamT& in = streams.in;
 		std::ostream& checkpointOut = streams.checkpointOut;
 
+		KmerHash contigEndKmers;
+		contigEndKmers.rehash((size_t)pow(2,28));
+
+		KmerHash visitedBranchKmers;
+
+		/*
+		 * Print a progress message whenever `READS_PROGRESS_STEP`
+		 * reads have been processed or `BASES_PROGRESS_STEP`
+		 * bases have been assembled. The purpose of using
+		 * two measures of progess is to show steady updates
+		 * throughout the assembly process.
+		 */
+
+		const size_t READS_PROGRESS_STEP = 100000;
+		const size_t BASES_PROGRESS_STEP = 1000000;
+		size_t basesProgressLine = BASES_PROGRESS_STEP;
+
 		while (true)
 		{
 			size_t readsUntilCheckpoint = params.readsPerCheckpoint;
@@ -939,13 +1009,34 @@ namespace BloomDBG {
 				for (std::vector<FastaRecord>::iterator it = buffer.begin();
 					 it != buffer.end(); ++it)
 				{
-					processRead(*it, goodKmerSet, assembledKmerSet,
-						params, counters, streams);
+					ReadRecord result =
+						processRead(*it, goodKmerSet, assembledKmerSet,
+							contigEndKmers, visitedBranchKmers,
+							params, counters, streams);
+
+#pragma omp critical(readsProgress)
+					{
+						++counters.readsProcessed;
+						if (params.verbose && counters.readsProcessed
+							% READS_PROGRESS_STEP == 0)
+							readsProgressMessage(counters);
+					}
+
+					if (params.verbose)
+#pragma omp critical(basesProgress)
+					{
+						if (counters.basesAssembled >= basesProgressLine) {
+							basesProgressMessage(counters);
+							while (counters.basesAssembled >= basesProgressLine)
+								basesProgressLine += BASES_PROGRESS_STEP;
+						}
+					}
+
+					if (!params.readLogPath.empty()) {
+#pragma omp critical(readLog)
+						streams.readLogOut << result;
+					}
 
-#pragma omp atomic
-					counters.readsProcessed++;
-					if (params.verbose && counters.readsProcessed % PROGRESS_STEP == 0)
-						printProgressMessage(counters);
 				}
 
 			} /* for batch of reads between I/O operations */
@@ -965,7 +1056,8 @@ namespace BloomDBG {
 		assert(in.eof());
 
 		if (params.verbose) {
-			printProgressMessage(counters);
+			readsProgressMessage(counters);
+			basesProgressMessage(counters);
 			std::cerr << "Assembly complete" << std::endl;
 		}
 
@@ -979,7 +1071,7 @@ namespace BloomDBG {
 	 * as an argument to the `breadthFirstSearch` function.
 	 */
 	template <typename GraphT>
-	class GraphvizBFSVisitor
+	class GraphvizBFSVisitor : public DefaultBFSVisitor<GraphT>
 	{
 		typedef typename boost::graph_traits<GraphT>::vertex_descriptor VertexT;
 		typedef typename boost::graph_traits<GraphT>::edge_descriptor EdgeT;
@@ -1001,31 +1093,21 @@ namespace BloomDBG {
 			m_out << "}\n";
 		}
 
-		/** Invoked when a vertex is initialized */
-		void initialize_vertex(const VertexT&, const GraphT&) {}
-
 		/** Invoked when a vertex is visited for the first time */
-		void discover_vertex(const VertexT& v, const GraphT&)
+		BFSVisitorResult discover_vertex(const VertexT& v, const GraphT&)
 		{
 			++m_nodesVisited;
 			/* declare vertex (GraphViz) */
 			m_out << '\t' << v.kmer().c_str() << ";\n";
+
+			return BFS_SUCCESS;
 		}
 
-		/** Invoked each time a vertex is visited */
-		void examine_vertex(const VertexT&, const GraphT&) {}
-
-		/**
-		 * Invoked when all of a vertex's outgoing edges have been
-		 * traversed.
-		 */
-		void finish_vertex(const VertexT&, const GraphT&) {}
-
 		/**
 		 * Invoked when an edge is traversed. (Each edge
 		 * in the graph is traversed exactly once.)
 		 */
-		void examine_edge(const EdgeT& e, const GraphT& g)
+		BFSVisitorResult examine_edge(const EdgeT& e, const GraphT& g)
 		{
 			++m_edgesVisited;
 			const VertexT& u = source(e, g);
@@ -1034,34 +1116,10 @@ namespace BloomDBG {
 			/* declare edge (GraphViz) */
 			m_out << '\t' << u.kmer().c_str() << " -> "
 				<< v.kmer().c_str() << ";\n";
+
+			return BFS_SUCCESS;
 		}
 
-		/**
-		 * Invoked when an edge is traversed to a "gray" vertex.
-		 * A vertex is gray when some but not all of its outgoing edges
-		 * have been traversed.
-		 */
-		void gray_target(const EdgeT&, const GraphT&) {}
-
-		/**
-		 * Invoked when an edge is traversed to a "black" vertex.
-		 * A vertex is black when all of its outgoing edges have
-		 * been traversed.
-		 */
-		void black_target(const EdgeT&, const GraphT&) {}
-
-		/**
-		 * Invoked when an edge is traversed to a "gray" or
-		 * "black" vertex.
-		 */
-		void non_tree_edge(const EdgeT&, const GraphT&) {}
-
-		/**
-		 * Invoked when an edge is traversed to a "white" vertex.
-		 * A vertex is a white if it is previously unvisited.
-		 */
-		void tree_edge(const EdgeT&, const GraphT&) {}
-
 		/** Return number of distinct nodes visited */
 		size_t getNumNodesVisited() const
 		{
@@ -1140,14 +1198,14 @@ namespace BloomDBG {
 				std::string startKmer = seq.substr(0, k);
 				Vertex start(startKmer.c_str(),
 					RollingHash(startKmer, numHashes, k));
-				breadthFirstSearch(dbg, start, visitor, colorMap);
+				breadthFirstSearch(start, dbg, colorMap, visitor);
 
 				/* BFS traversal in reverse dir */
 				Sequence rcSeq = reverseComplement(seq);
 				std::string rcStartKmer = rcSeq.substr(0, k);
 				Vertex rcStart(rcStartKmer.c_str(),
 					RollingHash(rcStartKmer, numHashes, k));
-				breadthFirstSearch(dbg, rcStart, visitor, colorMap);
+				breadthFirstSearch(rcStart, dbg, colorMap, visitor);
 
 			}
 
