@@ -641,27 +641,5 @@ int main(int argc, char** argv)
 		printContiguityStats(cerr, lengthHistogram, STATS_MIN_LENGTH)
 			<< '\t' << opt::out << '\n';
 	}
-#if 0
-	// assembly contiguity statistics
-	vector<int> vals = passContiguityStatsVal(lengthHistogram,200);
-	vector<string> keys = make_vector<string>()
-		<< "n"
-		<< "n200"
-		<< "nN50"
-		<< "min"
-		<< "N80"
-		<< "N50"
-		<< "N20"
-		<< "Esize"
-		<< "max"
-		<< "sum"
-		<< "nNG50"
-		<< "NG50";
-
-	if (!opt::db.empty()) {
-		for (unsigned a=0; a<vals.size(); a++)
-			addToDb(db, keys[a], vals[a]);
-	}
-#endif
 	return 0;
 }
