@@ -46,6 +46,7 @@
 #endif
 
 using namespace std;
+using Konnector::BloomFilter;
 #if USESEQAN
 using namespace seqan;
 #endif
@@ -78,11 +79,10 @@ static const char USAGE_MESSAGE[] =
 "  -b, --bloom-size=N           size of Bloom filter (e.g. '40G'). Required\n"
 "                               when not using pre-built Bloom filter(s)\n"
 "                               (-i option)\n"
-"  -B, --max-branches=N         max branches in de Bruijn graph traversal;\n"
-"                               use 'nolimit' for no limit [1000]\n"
 "  -d, --dot-file=FILE          write graph traversals to a DOT file\n"
 "  -e, --fix-errors             find and fix single-base errors when reads\n"
 "                               have no kmers in bloom filter [disabled]\n"
+"  -C, --max-cost=N             max edges to traverse during each graph search [100000]\n"
 "  -i, --input-bloom=FILE       load bloom filter from FILE\n"
 "      --mask                   mask new and changed bases as lower case\n"
 "      --no-mask                do not mask bases [default]\n"
@@ -110,6 +110,7 @@ static const char USAGE_MESSAGE[] =
 "  -s, --search-mem=N           mem limit for graph searches; multiply by the\n"
 "                               number of threads (-j) to get the total mem used\n"
 "                               for graph traversal [500M]\n"
+"  -g, --gap-file=FILE          write sealed gaps to FILE\n"
 "  -t, --trace-file=FILE        write graph search stats to FILE\n"
 "  -v, --verbose                display verbose output\n"
 "      --help                   display this help and exit\n"
@@ -117,19 +118,20 @@ static const char USAGE_MESSAGE[] =
 "\n"
 " Deprecated Options:\n"
 "\n"
+"  -B, --max-branches=N         max branches in de Bruijn graph traversal;\n"
+"                               use 'nolimit' for no limit [nolimit]\n"
 "  -f, --min-frag=N             min fragment size in base pairs\n"
 "  -F, --max-frag=N             max fragment size in base pairs\n"
 "\n"
-"   Note: --max-frag was formerly used to determine the maximum gap\n"
-"   size that abyss-sealer would attempt to close, according to the formula\n"
-"   max_gap_size = max_frag - 2 * flank_length, where flank_length is\n"
-"   deteremined by the -L option.  --max-frag is kept only for backwards\n"
-"   compatibility and is superceded by the more intuitive -G (--max-gap-length)\n"
-"   option. Similarly, --min-frag determines the minimum gap size to close,\n"
-"   according to the formula min_gap_size = min_frag - 2 * flank_length, where\n"
-"   a negative gap size indicates an overlap between gap flanks.  Normally the\n"
-"   user would not want to specify a minimum gap size and so it is recommended to\n"
-"   leave --min-frag unset.\n"
+"  Note 1: --max-branches was not effective for truncating expensive searches,\n"
+"  and has been superceded by the --max-cost option.\n"
+"\n"
+"  Note 2: --max-frag was formerly used to determine the maximum gap\n"
+"  size that abyss-sealer would attempt to close, according to the formula\n"
+"  max_gap_size = max_frag - 2 * flank_length, where flank_length is\n"
+"  determined by the -L option.  --max-frag is superceded by the more\n"
+"  intuitive -G (--max-gap-length) option. The related option --min-frag\n"
+"  does not seem to have any practical use.\n"
 "\n"
 "Report bugs to <" PACKAGE_BUGREPORT ">.\n";
 
@@ -157,7 +159,14 @@ namespace opt {
 	bool interleaved = false;
 
 	/** Max active branches during de Bruijn graph traversal */
-	unsigned maxBranches = 1000;
+	unsigned maxBranches = NO_LIMIT;
+
+	/**
+	 * Max cost for a connecting path search. Searches that
+	 * exceed this cost limit will be aborted and the gap
+	 * will remain unfilled.
+	 */
+	unsigned maxCost = 100000;
 
 	/** multi-graph DOT file containing graph traversals */
 	static string dotPath;
@@ -204,6 +213,9 @@ namespace opt {
 	/** Output file for graph search stats */
 	static string tracefilePath;
 
+	/** Output file for sealed gaps */
+	static string gapfilePath;
+
 	/** Mask bases not in flanks */
 	static int mask = 0;
 
@@ -229,6 +241,7 @@ struct Counters {
 	size_t tooManyMismatches;
 	size_t tooManyReadMismatches;
 	size_t containsCycle;
+	size_t maxCostExceeded;
 	size_t exceededMemLimit;
 	size_t traversalMemExceeded;
 	size_t readPairsProcessed;
@@ -236,7 +249,7 @@ struct Counters {
 	size_t skipped;
 };
 
-static const char shortopts[] = "S:L:b:B:d:ef:F:G:i:Ij:k:lm:M:no:P:q:r:s:t:v";
+static const char shortopts[] = "S:L:b:B:C:d:ef:F:G:g:i:Ij:k:lm:M:no:P:q:r:s:t:v";
 
 enum { OPT_HELP = 1, OPT_VERSION };
 
@@ -248,6 +261,7 @@ static const struct option longopts[] = {
 	{ "max-gap-length",   required_argument, NULL, 'G' },
 	{ "bloom-size",       required_argument, NULL, 'b' },
 	{ "max-branches",     required_argument, NULL, 'B' },
+	{ "max-cost",         required_argument, NULL, 'C' },
 	{ "dot-file",         required_argument, NULL, 'd' },
 	{ "fix-errors",       no_argument, NULL, 'e' },
 	{ "min-frag",         required_argument, NULL, 'f' },
@@ -273,6 +287,7 @@ static const struct option longopts[] = {
 	{ "read-name",        required_argument, NULL, 'r' },
 	{ "search-mem",       required_argument, NULL, 's' },
 	{ "trace-file",       required_argument, NULL, 't' },
+	{ "gap-file",         required_argument, NULL, 'g' },
 	{ "verbose",          no_argument, NULL, 'v' },
 	{ "help",             no_argument, NULL, OPT_HELP },
 	{ "version",          no_argument, NULL, OPT_VERSION },
@@ -415,8 +430,8 @@ template <typename Graph>
 string merge(const Graph& g,
 	unsigned k,
 	const Gap& gap,
-	FastaRecord &read1,
-	FastaRecord &read2,
+	const FastaRecord &read1,
+	const FastaRecord &read2,
 	const ConnectPairsParams& params,
 	Counters& g_count,
 	ofstream& traceStream)
@@ -483,6 +498,11 @@ string merge(const Graph& g,
 			++g_count.containsCycle;
 			break;
 
+		case MAX_COST_EXCEEDED:
+#pragma omp atomic
+			++g_count.maxCostExceeded;
+			break;
+
 		case EXCEEDED_MEM_LIMIT:
 #pragma omp atomic
 			++g_count.exceededMemLimit;
@@ -572,12 +592,12 @@ void kRun(const ConnectPairsParams& params,
 	map<FastaRecord, map<FastaRecord, Gap> > &flanks,
 	unsigned &gapsclosed,
 	ofstream &logStream,
-	ofstream &traceStream)
+	ofstream &traceStream,
+	ofstream &gapStream)
 {
 	map<FastaRecord, map<FastaRecord, Gap> >::iterator read1_it;
 	map<FastaRecord, Gap>::iterator read2_it;
 	unsigned uniqueGapsClosed = 0;
-	bool success;
 
 	Counters g_count;
 	g_count.noStartOrGoalKmer = 0;
@@ -589,6 +609,7 @@ void kRun(const ConnectPairsParams& params,
 	g_count.tooManyMismatches = 0;
 	g_count.tooManyReadMismatches = 0;
 	g_count.containsCycle = 0;
+	g_count.maxCostExceeded = 0;
 	g_count.exceededMemLimit = 0;
 	g_count.traversalMemExceeded = 0;
 	g_count.readPairsProcessed = 0;
@@ -597,31 +618,50 @@ void kRun(const ConnectPairsParams& params,
 
 	printLog(logStream, "Flanks inserted into k run = " + IntToString(flanks.size()) + "\n");
 
-	for (read1_it = flanks.begin(); read1_it != flanks.end();) {
-		success = false;
+	int counter = 0;
+	vector<map<FastaRecord, map<FastaRecord, Gap> >::iterator> flanks_closed;
+#pragma omp parallel private(read1_it, read2_it) firstprivate(counter)
+	for (read1_it = flanks.begin(); read1_it != flanks.end(); ++read1_it) {
 		FastaRecord read1 = read1_it->first;
-		for (read2_it = flanks[read1].begin(); read2_it != flanks[read1].end(); read2_it++) {
+		bool success = false;
+		for (read2_it = flanks[read1].begin(); read2_it != flanks[read1].end(); ++read2_it, ++counter) {
+#if _OPENMP
+			if (counter % omp_get_num_threads() != omp_get_thread_num())
+				continue;
+#endif
 			FastaRecord read2 = read2_it->first;
 
 			int startposition = read2_it->second.gapStart();
 			string tempSeq = merge(g, k, read2_it->second, read1, read2, params, g_count, traceStream);
 			if (!tempSeq.empty()) {
 				success = true;
+#pragma omp critical (allmerged)
 				allmerged[read1.id.substr(0,read1.id.length()-2)][startposition]
 					= ClosedGap(read2_it->second, tempSeq);
-//#pragma omp atomic
-				gapsclosed++;
-//#pragma omp atomic
-				uniqueGapsClosed++;
-				if (gapsclosed % 100 == 0)
+#pragma omp atomic
+				++uniqueGapsClosed;
+#pragma omp critical (gapsclosed)
+				if (++gapsclosed % 100 == 0)
 					printLog(logStream, IntToString(gapsclosed) + " gaps closed so far\n");
+
+				if (!opt::gapfilePath.empty())
+#pragma omp critical (gapStream)
+					gapStream << ">" << read1.id.substr(0,read1.id.length()-2)
+						  << "_" << read2_it->second.gapStart() << "-" << read2_it->second.gapEnd()
+						  << " LN:i:" << tempSeq.length() << '\n'
+						  << tempSeq << '\n';
 			}
 		}
 		if (success) {
-			flanks.erase(read1_it++);
+#pragma omp critical (flanks_closed)
+			flanks_closed.push_back(read1_it);
 		}
-		else
-			read1_it++;
+	}
+
+	for (vector<map<FastaRecord, map<FastaRecord, Gap> >::iterator>::iterator it = flanks_closed.begin();
+			it != flanks_closed.end(); ++it) {
+		if (flanks.count((*it)->first) > 0)
+		    flanks.erase(*it);
 	}
 
 	printLog(logStream, IntToString(uniqueGapsClosed) + " unique gaps closed for k" + IntToString(k) + "\n");
@@ -644,6 +684,8 @@ void kRun(const ConnectPairsParams& params,
 		+ sizetToString(g_count.tooManyReadMismatches) + "\n");
 	printLog(logStream, "Contains cycle: "
 		+ sizetToString(g_count.containsCycle) + "\n");
+	printLog(logStream, "Max cost exceeded: "
+		+ sizetToString(g_count.maxCostExceeded) + "\n");
 	printLog(logStream, "Exceeded mem limit: "
 		+ sizetToString(g_count.exceededMemLimit) + "\n");
 	printLog(logStream, "Skipped: "
@@ -723,6 +765,8 @@ int main(int argc, char** argv)
 			opt::bloomSize = SIToBytes(arg); break;
 		  case 'B':
 			setMaxOption(opt::maxBranches, arg); break;
+		  case 'C':
+			setMaxOption(opt::maxCost, arg); break;
 		  case 'd':
 			arg >> opt::dotPath; break;
 		  case 'e':
@@ -771,6 +815,8 @@ int main(int argc, char** argv)
 			opt::searchMem = SIToBytes(arg); break;
 		  case 't':
 			arg >> opt::tracefilePath; break;
+		  case 'g':
+		    arg >> opt::gapfilePath; break;
 		  case 'v':
 			opt::verbose++; break;
 		  case OPT_HELP:
@@ -874,6 +920,13 @@ int main(int argc, char** argv)
 		assert_good(traceStream, opt::tracefilePath);
 	}
 
+	ofstream gapStream;
+	if (!opt::gapfilePath.empty()) {
+		gapStream.open(opt::gapfilePath.c_str());
+		assert(gapStream.is_open());
+		assert_good(gapStream, opt::gapfilePath);
+	}
+
 	string logOutputPath(opt::outputPrefix);
 	logOutputPath.append("_log.txt");
 	ofstream logStream(logOutputPath.c_str());
@@ -897,6 +950,7 @@ int main(int argc, char** argv)
 	params.maxMergedSeqLen = opt::maxGapLength + 2 * opt::flankLength;
 	params.maxPaths = opt::maxPaths;
 	params.maxBranches = opt::maxBranches;
+	params.maxCost = opt::maxCost;
 	params.maxPathMismatches = opt::maxMismatches;
 	params.maxReadMismatches = opt::maxFlankMismatches;
 	/*
@@ -1001,10 +1055,10 @@ int main(int argc, char** argv)
 #ifdef _OPENMP
 			ConcurrentBloomFilter<CascadingBloomFilter> cbf(*cascadingBloom, 1000);
 			for (int i = optind; i < argc; i++)
-				Bloom::loadFile(cbf, opt::k, argv[i], 0 /*opt::verbose*/);
+				Bloom::loadFile(cbf, opt::k, argv[i], opt::verbose >= 2);
 #else
 			for (int i = optind; i < argc; i++)
-				Bloom::loadFile(*cascadingBloom, opt::k, argv[i], 0 /* opt::verbose*/);
+				Bloom::loadFile(*cascadingBloom, opt::k, argv[i], opt::verbose >= 2);
 #endif
 			bloom = &cascadingBloom->getBloomFilter(opt::max_count - 1);
 		}
@@ -1020,7 +1074,7 @@ int main(int argc, char** argv)
 		temp = "Starting K run with k = " + IntToString(opt::k) + "\n";
 		printLog(logStream, temp);
 
-		kRun(params, opt::k, g, allmerged, flanks, gapsclosed, logStream, traceStream);
+		kRun(params, opt::k, g, allmerged, flanks, gapsclosed, logStream, traceStream, gapStream);
 
 		temp = "k" + IntToString(opt::k) + " run complete\n"
 				+ "Total gaps closed so far = " + IntToString(gapsclosed) + "\n\n";
@@ -1073,5 +1127,10 @@ int main(int argc, char** argv)
 		traceStream.close();
 	}
 
+	if (!opt::gapfilePath.empty()) {
+		assert_good(gapStream, opt::gapfilePath);
+		gapStream.close();
+	}
+
 	return 0;
 }
