@@ -8,7 +8,7 @@
 using namespace std;
 using namespace boost;
 
-typedef RollingBloomDBG<BTL::BloomFilter> Graph;
+typedef RollingBloomDBG<BloomFilter> Graph;
 typedef graph_traits<Graph> GraphTraits;
 typedef graph_traits<Graph>::vertex_descriptor V;
 
@@ -20,7 +20,7 @@ protected:
 	const unsigned m_k;
 	const unsigned m_bloomSize;
 	const unsigned m_numHashes;
-	BTL::BloomFilter m_bloom;
+	BloomFilter m_bloom;
 	Graph m_graph;
 
 	RollingBloomDBGTest() : m_k(5), m_bloomSize(100000), m_numHashes(2),
@@ -149,7 +149,7 @@ TEST_F(RollingBloomDBGTest, pathTraversal)
 	 * CGACT-GACTC-ACTCG
 	 */
 
-	BTL::BloomFilter bloom(m_bloomSize, m_numHashes, m_k);
+	BloomFilter bloom(m_bloomSize, m_numHashes, m_k);
 	Graph graph(bloom);
 
 	const V CGACT("CGACT", RollingHash("CGACT", m_numHashes, m_k));
@@ -189,6 +189,43 @@ TEST_F(RollingBloomDBGTest, pathTraversal)
 	ASSERT_EQ(ei_end, ei);
 }
 
+TEST_F(RollingBloomDBGTest, vertexComparison)
+{
+	/*
+	 * Note: vertex comparision should be
+	 * invariant with respect to k-mer orientation.
+	 * In other words, a k-mer and its reverse
+	 * complement are considered to be the same
+	 * vertex. Different vertices are compared
+	 * by converting them to their canonical
+	 * orientations.
+	 */
+
+	const V kmer1("ACGTA", RollingHash("ACGTA", m_numHashes, m_k));
+	const V rcKmer1("TACGT", RollingHash("TACGT", m_numHashes, m_k));
+
+	const V kmer2("TGCAT", RollingHash("TGCAT", m_numHashes, m_k));
+	const V rcKmer2("ATGCA", RollingHash("ATGCA", m_numHashes, m_k));
+
+	ASSERT_TRUE(kmer1.isCanonical());
+	ASSERT_FALSE(kmer2.isCanonical());
+
+	/* we expect a vertex to be equal to its reverse complement */
+
+	ASSERT_EQ(kmer1, rcKmer1);
+	ASSERT_EQ(kmer2, rcKmer2);
+
+	/*
+	 * we expect kmer1 < kmer2, regardless of the orientation
+	 * of the two k-mers
+	 */
+
+	ASSERT_TRUE(kmer1 < kmer2);
+	ASSERT_TRUE(kmer1 < rcKmer2);
+	ASSERT_TRUE(rcKmer1 < kmer2);
+	ASSERT_TRUE(rcKmer1 < rcKmer2);
+}
+
 /** Test fixture for RollingBloomDBG with spaced seed k-mers. */
 class RollingBloomDBGSpacedSeedTest : public ::testing::Test
 {
@@ -197,7 +234,7 @@ protected:
 	const unsigned m_k;
 	const unsigned m_bloomSize;
 	const unsigned m_numHashes;
-	BTL::BloomFilter m_bloom;
+	BloomFilter m_bloom;
 	Graph m_graph;
 	const std::string m_spacedSeed;
 
